/* Compal Laptop Control
 * Author: Krzemin <pkrzemin@o2.pl>
 * WWW: http://krzemin.iglu.cz
 * This software is distributed under the GNU General Public License.
 */

#include "frmcompal.h"


frmCompal::frmCompal(QWidget * parent, Qt::WFlags f) : QWidget(parent, f)
{
	ui.setupUi(this);

	if(!(inited = initCompal()))
	{
		return;
	}

	if(QSystemTrayIcon::isSystemTrayAvailable())
	{
		QPixmap pixmap(":/mask/mask.png");
		setMask(pixmap.mask());
	}

	connect (ui.cbBluetooth, SIGNAL(toggled(bool)), this, SLOT(cbBluetoothToggled(bool)));
	connect (ui.cbWifi, SIGNAL(toggled(bool)), this, SLOT(cbWifiToggled(bool)));
	connect (ui.hsBrightness, SIGNAL(valueChanged(int)), this, SLOT(hsBrightnessChanged(int)));
	connect (ui.hsBatteryCL, SIGNAL(valueChanged(int)), this, SLOT(hsBatteryCLChanged(int)));

	setMouseTracking(true);

	if(GlobalConfig().restoreSettings())
	{
		if(compal.isBtAvailable())
		{
			ui.cbBluetooth->setChecked(GlobalConfig().bluetoothOn());
			cbBluetoothToggled(ui.cbBluetooth->isChecked());
		}
		if(compal.isWifiAvailable())
		{
			ui.cbWifi->setChecked(GlobalConfig().wlanOn());
			cbWifiToggled(ui.cbWifi->isChecked());
		}
		if(compal.isLcdAvailable())
		{
			ui.hsBrightness->setValue(GlobalConfig().brightness());
			hsBrightnessChanged(ui.hsBrightness->value());
		}
		if(compal.isBatteryCLAvailable())
		{
			ui.hsBatteryCL->setValue(GlobalConfig().batteryCL());
			hsBatteryCLChanged(ui.hsBatteryCL->value());
		}
	}

	setWindowOpacity(GlobalConfig().opacity() ? 0.85 : 1);

	updateTimer = new QTimer(this);
	connect(updateTimer, SIGNAL(timeout()), this, SLOT(updateUiStatus()));
	updateTimer->start(500);
}

frmCompal::~frmCompal()
{}

bool frmCompal::initCompal()
{
	if(!compal.init())
	{
		QMessageBox::critical(	this,
								tr("Error"),
								QString("%1\n\n%2")
									.arg(compal.lastError())
									.arg(tr("See %1/README for details.").arg(DOC_DIR)));
		return false;
	}
	
	if(!compal.isBtAvailable())
	{
		ui.cbBluetooth->setDisabled(true);
		ui.cbBluetooth->setToolTip( QString("%1\n\n%2")
										.arg(tr("You can't control bluetooth power, because you haven't "
												"such permissions to\n%1 file."))
										.arg(compal.getBtFile()).arg(tr("See %1/README for details.").arg(DOC_DIR)));
	}
	
	if(!compal.isWifiAvailable())
	{
		ui.cbWifi->setDisabled(true);
		ui.cbWifi->setToolTip( QString("%1\n\n%2")
								.arg(tr("You can't control wireless lan power, because you haven't "
										"such permissions to\n%1 file."))
								.arg(compal.getWifiFile()).arg(tr("See %1/README for details.").arg(DOC_DIR)));
	}
	
	if(!compal.isLcdAvailable())
	{
		QString msg = QString("%1\n\n%2")
						.arg(tr("You can't control lcd brightness, because you haven't "
								"such permissions to\n%1 file."))
						.arg(compal.getLcdFile()).arg(tr("See %1/README for details.").arg(DOC_DIR));
		
		ui.hsBrightness->setDisabled(true);
		ui.lbBrightness->setDisabled(true);
		ui.lbBrightnessP->setDisabled(true);
		ui.hsBrightness->setToolTip(msg);
		ui.lbBrightness->setToolTip(msg);
		ui.lbBrightnessP->setToolTip(msg);
		ui.lbBrightnessP->setText("???");
	}
	else
	{
		int max = compal.lcdMax();
		if(max > 0)
		{
			ui.hsBrightness->setMaximum(max);
		}
	}
	
	if(!compal.isBatteryCLAvailable())
	{
		QString msg = QString("%1\n\n%2")
						.arg(tr("You can't control battery charging level, because you haven't "
								"such permissions to\n%1 file."))
						.arg(compal.getBatteryCLFile()).arg(tr("See %1/README for details.").arg(DOC_DIR));
		
		ui.hsBatteryCL->setDisabled(true);
		ui.lbBatteryCL->setDisabled(true);
		ui.lbBatteryCLP->setDisabled(true);
		ui.hsBatteryCL->setToolTip(msg);
		ui.lbBatteryCL->setToolTip(msg);
		ui.lbBatteryCLP->setToolTip(msg);
		ui.lbBatteryCLP->setText("???");
	}
	
	return true;
}

void frmCompal::createTrayIcon()
{
	showAction = new QAction(tr("Show"), this);
	connect(showAction, SIGNAL(triggered()), this, SLOT(showHideWindow()));
	
	restoreAction = new QAction(tr("Save and restore settings"), this);
	restoreAction->setCheckable(true);
	restoreAction->setChecked(GlobalConfig().restoreSettings());
	connect(restoreAction, SIGNAL(triggered()), this, SLOT(restoreClicked()));
	
	opacityAction = new QAction(tr("Opacity window"), this);
	opacityAction->setCheckable(true);
	opacityAction->setChecked(GlobalConfig().opacity());
	connect(opacityAction, SIGNAL(triggered()), this, SLOT(opacityClicked()));
	
	aboutAction = new QAction(tr("About..."), this);
	connect(aboutAction, SIGNAL(triggered()), this, SLOT(showAbout()));
	
	quitAction = new QAction(tr("Quit"), this);
	connect(quitAction, SIGNAL(triggered()), qApp, SLOT(quit()));
	
	trayIconMenu = new QMenu(this);
	trayIconMenu->addAction(showAction);
	trayIconMenu->addAction(restoreAction);
	trayIconMenu->addAction(opacityAction);
	trayIconMenu->addAction(aboutAction);
	trayIconMenu->addSeparator();
	trayIconMenu->addAction(quitAction);
	
	trayIcon = new QSystemTrayIcon(this);
	trayIcon->setContextMenu(trayIconMenu);
	trayIcon->setIcon(QIcon(":/icon/compal-laptop-control-icon.png"));
	
	connect(trayIcon, SIGNAL(activated(QSystemTrayIcon::ActivationReason)),
			this, SLOT(iconActivated(QSystemTrayIcon::ActivationReason)));
	
	connect(this, SIGNAL(toolTipChange(const QString &)), this, SLOT(setToolTip(const QString &)));
	
	trayIcon->show();
}

void frmCompal::setToolTip(const QString & tip)
{
	trayIcon->setToolTip(tip);
}

void frmCompal::updateUiStatus()
{
	static bool bt, wifi;
	static int lcd, battery;
	
	bt = compal.btStatus() > 0;
	wifi = compal.wifiStatus() > 0;
	lcd = compal.lcdStatus();
	battery = compal.batteryCLStatus();
	
	ui.cbBluetooth->setChecked(bt);
	ui.cbWifi->setChecked(wifi);
	ui.hsBrightness->setSliderPosition(lcd);
	if(!ui.hsBatteryCL->isSliderDown())
		ui.hsBatteryCL->setSliderPosition(battery > 0 ? battery : 100);
	
	if(compal.isBtAvailable())
		GlobalConfig().setBluetoothOn(bt);
	if(compal.isWifiAvailable())
		GlobalConfig().setWlanOn(wifi);
	if(compal.isLcdAvailable())
		GlobalConfig().setBrightness(lcd);
	if(compal.isBatteryCLAvailable())
		GlobalConfig().setBatteryCL(battery);
	GlobalConfig().save();
	
	QString toolTipStr = tr("Bluetooth") + QString(": ") + (bt ? tr("On") : tr("Off")) + "\n";
	toolTipStr += tr("Wireless LAN") + QString(": ") + (wifi ? tr("On") : tr("Off")) + "\n";
	toolTipStr += tr("LCD brightness:") + QString(" %1%").arg(lcd * 100 / 7);
	
	if(compal.isBatteryCLAvailable())
	{
		toolTipStr += QString("\n") + tr("Battery charging level:") + QString(" %1%").arg(battery);
	}
	
	emit toolTipChange(toolTipStr);
}

void frmCompal::cbBluetoothToggled(bool checked)
{
	compal.setBtStatus(checked);
}

void frmCompal::cbWifiToggled(bool checked)
{
	compal.setWifiStatus(checked);
}

void frmCompal::hsBrightnessChanged(int value)
{
	compal.setLcdStatus(value);
	ui.lbBrightnessP->setText(QString("%1%").arg(value * 100 / compal.lcdMax()));
}

void frmCompal::hsBatteryCLChanged(int value)
{	
	compal.setBatteryCLStatus(value);
	ui.lbBatteryCLP->setText(QString("%1%").arg(value));
}

void frmCompal::iconActivated(QSystemTrayIcon::ActivationReason reason)
{
	if(reason == QSystemTrayIcon::Trigger)
	{
		showHideWindow();
	}
}

void frmCompal::showHideWindow()
{
	int SW = QApplication::desktop()->width(),
		SH = QApplication::desktop()->height();
	
	move(QCursor::pos().x() - width() / 2, QCursor::pos().y() - height() - 10);
	
	int x = pos().x(),
		y = pos().y();
	
	const int SPACING = 30;
	
	if (x <= SPACING ) x = SPACING;
	if (y <= SPACING ) y = SPACING;
	if (x + width() > SW - SPACING ) x = SW - width() - SPACING;
	if (y + height() > SH - SPACING ) y = SH - height() - SPACING;
	
	move(x, y);
	
	isVisible() ? hide() : show();
}

void frmCompal::showAbout()
{
	static frmAbout *about;
	
	if(!about)
	{
		about = new	frmAbout();
	}
	
	if(about->isVisible())
	{
		about->raise();
		return;
	}
	
	about->exec();
	delete about;
	about = 0;
}

void frmCompal::restoreClicked()
{
	GlobalConfig().setRestoreSettings(restoreAction->isChecked());
	GlobalConfig().save();
}

void frmCompal::opacityClicked()
{
	setWindowOpacity(opacityAction->isChecked() ? 0.85 : 1);
	GlobalConfig().setOpacity(opacityAction->isChecked());
	GlobalConfig().save();
}

void frmCompal::keyPressEvent(QKeyEvent *event)
{
	if(ui.hsBrightness->isEnabled())
	{
		int change = 0;
		if( (event->key() == Qt::Key_Left) || (event->key() == Qt::Key_Down) )
		{
			change = -1;
		}
		else if( (event->key() == Qt::Key_Right) || (event->key() == Qt::Key_Up) )
		{
			change = 1;
		}
		
		ui.hsBrightness->setValue(ui.hsBrightness->value() + change);
	}
	
	if(ui.cbBluetooth->isEnabled() && (event->key() == Qt::Key_B))
	{
		ui.cbBluetooth->setChecked(!ui.cbBluetooth->isChecked());
		cbBluetoothToggled(ui.cbBluetooth->isChecked());
	}
	
	if(ui.cbWifi->isEnabled() && (event->key() == Qt::Key_W))
	{
		ui.cbWifi->setChecked(!ui.cbWifi->isChecked());
		cbWifiToggled(ui.cbWifi->isChecked());
	}
}

void frmCompal::paintEvent(QPaintEvent *event)
{
	if(QSystemTrayIcon::isSystemTrayAvailable())
	{
		QPainter painter(this);
		painter.setPen(palette().color(QPalette::WindowText));
		painter.drawPolygon(borderPoints, 8);
		event->accept();
	}
}
